#include <stdio.h>
#include <stdlib.h>
#include <termios.h>
#include <unistd.h>  // usleep 
#include <fcntl.h>   // file serial stuff 
#include <sys/ioctl.h>
#include "usb_i2c.h"
#include "md23.h"

// This prints out encoder counts and motor current.
// The current is only to 0.1A resolution!  You may need to rub the wheel
// pretty hard to get >100mA so that it'll show up.
// KMN 7/18/2007

void SetSpeed(BYTE addr, BYTE speed1, BYTE speed2) {
	BYTE *sbuf;
	int enc1=0;
	int i;
	BYTE m1current;

	// Verify the address by reading the rev
	printf("Checking address 0x%02X for MD23 ...",addr);
	sbuf = Read_i2c(addr,MD23_REVISION,1);
	if(sbuf[0]>0 && sbuf[0]<20) { // found one!
			printf("found one, revision %i!\n",sbuf[0]);
	}
	free(sbuf);

 	printf("Reset Encoders...");  fflush(stdout);
	Write_i2c(addr,MD23_COMMAND,CLEAR_ENCS);
 	printf("done.\n");  fflush(stdout);

	printf("Set Mode to Signed...");  fflush(stdout);
	Write_i2c(addr,MODE,1);
 	printf("done.\n");  fflush(stdout);

	printf("Sending Speeds to MD23 @ %02x .. ",addr);  fflush(stdout);
	Write_i2c(addr,SPEED1,speed1);
	Write_i2c(addr,SPEED2,0);
 	printf("done.\n");  fflush(stdout);

	printf("Starting motor...\n");  fflush(stdout);
	while (enc1<1000) {
			  static int oldenc1;

// Print all registers, in hex
//
//			  sbuf = Read_i2c(addr,0,17);
//			  if (sbuf == NULL) return;
//			  for (i=0;i<17;i++) printf("sbuf[%i]=%02x\n",i,sbuf[i]);
//			  free(sbuf);

			  sbuf = Read_i2c(addr,ENC1A,8);
			  if (sbuf == NULL) return;
			  enc1 = (sbuf[0]<< 3*8) +
						(sbuf[1]<< 2*8) +
						(sbuf[2]<< 1*8) +
						sbuf[3];
			  free(sbuf);

			  sbuf = Read_i2c(addr,M1CURRENT,1);
			  if (sbuf == NULL) return;
			  m1current = sbuf[0];
			  free(sbuf);

			  fprintf(fp,"%i %i\n",enc1,m1current);
			  printf("enc1=%i m1curr=%i mA\n",enc1,100*m1current);
			  usleep(1000*1000);
			  printf("\n%i",enc1);fflush(stdout);
	}
	fclose(fp);
 	printf("done.\n");  fflush(stdout);

	// Stop
 	printf("Stopping Motor.\n");  fflush(stdout);
	Write_i2c(addr,SPEED1,0);
	Write_i2c(addr,SPEED2,0);
 	printf("done.\n");  fflush(stdout);
}


int main (int argc, char *argv[]) {
 BYTE addr=0xb4;

 if (SerialOpen("/dev/ttyUSB0",B19200)<0)
	if (SerialOpen("/dev/ttyUSB1",B19200)<0) {
			fprintf(stderr,"Problem opening port");
			return -1;
 }

 SetSpeed(addr,11,0);
 return 0;
} 
