#include <stdio.h>
#include <stdlib.h>
#include <termios.h>
#include <unistd.h>  // usleep 
#include <fcntl.h>   // file serial stuff 
#include <sys/ioctl.h>
#include "usb_i2c.h"
#include "md23.h"

// This finds and re-sets the address of an MD23 on the USB-I2C Bus.
// usage:  ./md23_newaddr b4 (or ./md23_newaddr 0xb4)
// KMN 7/15/2007
	 
void SetAddr(BYTE newaddr) {
	const BYTE addr_low = 0xb0;
	const BYTE addr_high = 0xbe;
	static BYTE Addr=0;
	BYTE *sbuf;
	BYTE search_addr = addr_low;

	// Get USB-I2C firmware revision number
	sbuf = usb_i2c_cmd(USB_I2C_REVISION,0,0);
	printf("USB-I2C rev %i.\n",sbuf[0]);
	free(sbuf);

	// Find the MD23 on bus
	Addr=0;
	while(Addr==0) {			
		printf("Checking address 0x%02X for MD23 ...",search_addr);
	   sbuf = Read_i2c(search_addr,MD23_REVISION,1);

		if(sbuf[0]>0 && sbuf[0]<20) { // found one!
			printf("found one, revision %i!\n",sbuf[0]);
			Addr = search_addr;
		}
		else {
		   printf("(got %i)\n",sbuf[0]);
			search_addr += 2;
			if(search_addr>addr_high) {
					  printf("\n");
					  search_addr = addr_low;
			}
		}
	}

	// If already know where MD23  is, 
		printf("Setting MD23  address of Addr=%02x to %02x.\n",Addr,newaddr);

		Write_i2c(Addr,MD23_COMMAND,0xA0);
		Write_i2c(Addr,MD23_COMMAND,0xAA);
		Write_i2c(Addr,MD23_COMMAND,0xA5);
		Write_i2c(Addr,MD23_COMMAND,newaddr);
		printf("Address set to %02x.\n",newaddr);
}


int main (int argc, char *argv[]) {
 BYTE newaddr=0x00;
 if (argc!=2) {
			fprintf(stderr,"usage: %s 0xXX, XX hex digits\n",argv[0]);
			return -1;
 }
 else {
			newaddr = strtol(argv[1],(char **)NULL,16);
			switch(newaddr) {
					  case 0xB0:
					  case 0xB2:
					  case 0xB4:
					  case 0xB6:
					  case 0xB8:
					  case 0xBA:
					  case 0xBC:
					  case 0xBE:
								 break;
					  default:
								 fprintf(stderr,"Address must be even 0xB0-0xBE\n");
								 return -1;
			}
			printf("setting MD23  address to %02x\n",newaddr);
 }

 if (SerialOpen("/dev/ttyUSB0",B19200)<0)
	if (SerialOpen("/dev/ttyUSB1",B19200)<0) {
			fprintf(stderr,"Problem opening port");
			return -1;
 }

  SetAddr(newaddr);
 return 0;
} 
