#include <stdio.h>
#include <stdlib.h>
#include <termios.h>
#include <unistd.h>  // usleep 
#include <fcntl.h>   // file serial stuff 
#include <sys/ioctl.h>
#include "usb_i2c.h"
#include "srf02.h"

// This finds an SRF02 on the I2C Bus and repeatedly polls it, with an
// EXPLICIT sleep between.  If more than one are on the bus, it'll find the
// lowest numbered one.
// KMN 7/16/2007

int ReadSRF02Stuff(void) {
	const BYTE addr_low = 0xe0;
	const BYTE addr_high = 0xfe;
	static BYTE Addr=0;
	BYTE *sbuf;
	BYTE search_addr = addr_low;
	int range;

	printf("\n");

	// Get USB-I2C firmware revision number
	sbuf = usb_i2c_cmd(USB_I2C_REVISION,0,0);
	printf("\tUSB-I2C rev %i ",sbuf[0]);
	free(sbuf);

	// Find the srf02 on bus
	//Addr=0;
	while (Addr==0) {			// find srf02
		printf("Checking address 0x%02X for SRF02 ...",search_addr);

	   sbuf = Read_i2c(search_addr,SRF02_COMMAND,1);
		if(sbuf[0]>0 && sbuf[0]<20) { // found one!
			printf("found one, revision %i!\n",sbuf[0]);
			Addr = search_addr;
		} else {
		   printf("(got %i)\n",sbuf[0]);
			search_addr += 2;
			printf("\n");
			if(search_addr>addr_high) {
					  printf("\n");
					  search_addr = addr_low;
			}
		}
	}

	// Now, we know where SRF02 is.  Read all relevant data
#if 0
	sbuf = Read_i2c(Addr,SRF02_COMMAND,6);
	printf("\tsrf02ver %i",sbuf[0]);
	fflush(stdout);

	if(sbuf[0]>20) {
		if (sbuf[0]==255) {
			printf("(not ready)\n");
			return -1;
		}else{
			fprintf(stderr,"Comm Error, start over");
			Addr=0;
			return -1;
		}
	}

	//printf("unused=%02x\t",sbuf[1]);
	range = (sbuf[2]<<8)|sbuf[3];
	//printf("range=%i us =%i cm =%i in   ",n,n/58,n/148);
	printf("\trange=%5i cm\t",range/58);
	//minrange = (sbuf[4]<<8)|sbuf[5];
	//printf("minrange=%5i\n",n);

	free(sbuf);
#endif
	sbuf = Read_i2c(Addr,SRF02_RANGEH,2);
	range = (sbuf[0]<<8)|sbuf[1];
	printf("\trange=%5i cm\t",range/58);
	free(sbuf);

   // Send another ping before we leave
	Write_i2c(Addr,0x00,RANGE_US);

	return range;
}


int main (void) {
		  int range;

 if (SerialOpen("/dev/ttyUSB0",B19200)<0)
	if (SerialOpen("/dev/ttyUSB1",B19200)<0) {
			fprintf(stderr,"Problem opening port");
			return -1;
 }

 while (1) {
 	 range = ReadSRF02Stuff();
	 usleep(65*1000);
	 }
}

