#include <stdio.h>
#include <stdlib.h>
#include <termios.h>
#include <unistd.h>  // usleep 
#include <fcntl.h>   // file serial stuff 
#include <sys/ioctl.h>
#include "usb_i2c.h"
#include "srf02.h"

// This finds and re-sets the address of an SRF02 on the USB-I2C Bus.
// usage:  ./srf02_newaddr e4 (or ./srf02_newaddr 0xe4)
// KMN 7/15/2007
	 

void SetAddr(BYTE newaddr) {
	const BYTE addr_low = 0xe0;
	const BYTE addr_high = 0xfe;
	static BYTE Addr=0;
	BYTE *sbuf;
	BYTE search_addr = addr_low;

	// Get USB-I2C firmware revision number
	sbuf = usb_i2c_cmd(USB_I2C_REVISION,0,0);
	printf("USB-I2C rev %i.\n",sbuf[0]);
	free(sbuf);


	Addr=0;
	while(Addr==0) {			// find srf02
		printf("Checking address 0x%02X for SRF02 ...",search_addr);
	   sbuf = Read_i2c(search_addr,SRF02_REVISION,1);

		if(sbuf[0]>0 && sbuf[0]<20) { // found one!
			printf("found one, revision %i!\n",sbuf[0]);
			Addr = search_addr;
		}
		else {
		   printf("(got %i)\n",sbuf[0]);
			search_addr += 2;
			if(search_addr>addr_high) {
					  printf("\n");
					  search_addr = addr_low;
			}
		}

	}

	// If already know where SRF02 is, 
	printf("Setting SRF02 address of Addr=%02x to %02x.\n",Addr,newaddr);

		Write_i2c(Addr,SRF02_COMMAND,0xA0);
		Write_i2c(Addr,SRF02_COMMAND,0xAA);
		Write_i2c(Addr,SRF02_COMMAND,0xA5);
		Write_i2c(Addr,SRF02_COMMAND,newaddr);
		printf("Address set to %02x.\n",newaddr);
}


int main (int argc, char *argv[]) {
 BYTE newaddr=0x00;
 if (argc!=2) {
			fprintf(stderr,"usage: %s 0xXX, XX hex digits\n",argv[0]);
			return -1;
 }
 else {
			newaddr = strtol(argv[1],(char **)NULL,16);
			switch(newaddr) {
					  case 0xE0:
					  case 0xE2:
					  case 0xE4:
					  case 0xE6:
					  case 0xE8:
					  case 0xEA:
					  case 0xEC:
					  case 0xEE:
					  case 0xF0:
					  case 0xF2:
					  case 0xF4:
					  case 0xF6:
					  case 0xF8:
					  case 0xFA:
					  case 0xFC:
					  case 0xFE:
								 break;
					  default:
								 fprintf(stderr,"Address must be even 0xE0-0xFE\n");
								 return -1;
			}
			printf("setting SRF02 address to %02x\n",newaddr);
 }

 if (SerialOpen("/dev/ttyUSB0",B19200)<0)
	if (SerialOpen("/dev/ttyUSB1",B19200)<0) {
			fprintf(stderr,"Problem opening port");
			return -1;
 }

  SetAddr(newaddr);
 return 0;
} 
