#include <stdio.h>
#include <termios.h>
#include <unistd.h>  // usleep 
#include <fcntl.h>   // file serial stuff 
#include <sys/ioctl.h>
#include "usb_i2c_srf02.h"

// This reads two SRF02s on the bus.  Explicit addresses are required.
// It doesn't use the abstracted usb_i2c stuff yet.
// KMN 7/12/2007

int SerialOpen( const char* portDevStr, int portBaud ) {
	mFD = open(portDevStr,O_RDWR|O_NOCTTY);
   if (mFD<0){
	  fprintf(stderr,"SetupCommPort: Could not open %s\n",portDevStr);
	  perror("SerialOpen()");
	  return 0;
	  }
	  else{
				 printf("Opened %s for read/write\n",portDevStr);
	   CommPort=1;
	  }

    	if (ioctl(mFD, TCGETA, &mData)) {
        	perror("Serial::open_port()");
        	return 0;
    	}
    
    if(tcgetattr(mFD, &mData)<0) { perror("tcgetattr"); return -1;}
    if (cfsetispeed(&mData, portBaud)<0) {perror("cfsetispeed"); return -1;}
    mData.c_cflag |= (CLOCAL | CREAD);
    mData.c_lflag &=~(ICANON|ECHO|ISIG);
    mData.c_cflag &=~PARENB;
    mData.c_cflag &=~CSTOPB;
    mData.c_cflag &=~CSIZE;
    mData.c_cflag |= CS8;
    mData.c_iflag  = IGNBRK | IGNPAR | INLCR;
    tcflush(mFD, TCIFLUSH);
    tcsetattr(mFD, TCSANOW, &mData);
    return 1;
}

	 
unsigned int ReadSRF02_us(BYTE addr) {
unsigned int range;
int n;
BYTE sbuf[100];

	if(mFD<0) {
		fprintf(stderr,"Please Set Com Port.\n");
		return 0;
	}	

		sbuf[0] = I2C_CMD;	// get srf02 version
		sbuf[1] = addr+READ;
		sbuf[2] = 0x00; // read from register 0
		sbuf[3] = 0x04; // read four data bytes
	   n = write(mFD, &sbuf, 4);
		if (n!=4) {
				  fprintf(stderr,"Wrote %d (instead of 4) bytes...\n",n);
				  return 0;
		}
		n = read(mFD, &sbuf, 4);
		if (n!=4) {
				  fprintf(stderr,"Read %d (instead of 4) bytes...\n",n);
				  return 0;
		}

		// Result packet - USB_I2C returns as single packet
		// sbuf[0] is SRF02 Software Revision
		// sbuf[1] is unused
		// sbuf[2] is high byte of range
		// sbuf[3] is low byte 
		n = (sbuf[2]<<8)|sbuf[3];
		//printf("%02x: range=%i us =%i cm =%i in   \n",addr,n,n/58,n/148);
		range = (unsigned) n;

		// Send another range command right away.
		sbuf[0] = I2C_CMD;	// send sonar ranging (uS) command
		sbuf[1] = addr+WRITE;
		sbuf[2] = 0x00; // command register (0)
		sbuf[3] = 0x01; // 1 data byte
		sbuf[4] = 0x52; // result in us
	   n = write(mFD, &sbuf, 5);
		if (n!=5) {
				  fprintf(stderr,"Wrote %d (instead of 5) bytes...\n",n);
				  return 0;
		}
		n = read(mFD, &sbuf, 1); // should be zero - check?
		if (n!=1) {
				  fprintf(stderr,"Read %d (instead of 1) bytes...\n",n);
				  return 0;
		}
		return range;
}
unsigned int ReadSRF02(BYTE addr)  {return ReadSRF02_us(addr);}
unsigned int ReadSRF02_cm(BYTE addr)  {return ReadSRF02_us(addr)/58;}
unsigned int ReadSRF02_in(BYTE addr)  {return ReadSRF02_us(addr)/148;}


int main (void) {
 mFD = -1;
 unsigned int range_e4 = 0;
 unsigned int range_e6 = 0;
 if (SerialOpen("/dev/ttyUSB0",B19200)<0)
	if (SerialOpen("/dev/ttyUSB1",B19200)<0) {
			fprintf(stderr,"Problem opening port");
			return -1;
 }
 while (1) {
	 range_e4 = ReadSRF02_cm(0xe4);
	 range_e6 = ReadSRF02_cm(0xe6);
	 printf("e4=%5i cm \te6=%5i cm\n",range_e4,range_e6);
	 usleep(650);
	 }
}

