//This should use the USBI2C to change the address of the SRF02 (should be
//same seqence as SRF08) but it doesn't work! -- KMN 7/16/2007
//
//Sending the four bytes one at a time works, though (see other program).
#include <stdio.h>
#include <stdlib.h>
#include <termios.h>
#include <unistd.h>  // usleep 
#include <fcntl.h>   // file serial stuff 
#include <sys/ioctl.h>
#include "usb_i2c_srf02.h"

int SerialOpen( const char* portDevStr, int portBaud ) {
	mFD = open(portDevStr,O_RDWR|O_NOCTTY);
    	if (mFD<0){
	  fprintf(stderr,"SetupCommPort: Could not open %s\n",portDevStr);
	  perror("SerialOpen()");
	  return 0;
	  }
	  else{
				 printf("Opened %s for read/write\n",portDevStr);
	   CommPort=1;
	  }

    	if (ioctl(mFD, TCGETA, &mData)) {
        	perror("Serial::open_port()");
        	return 0;
    	}
    
    if(tcgetattr(mFD, &mData)<0) { perror("tcgetattr"); return -1;}
    if (cfsetispeed(&mData, portBaud)<0) {perror("cfsetispeed"); return -1;}
    mData.c_cflag |= (CLOCAL | CREAD);
    mData.c_lflag &=~(ICANON|ECHO|ISIG);
    mData.c_cflag &=~PARENB;
    mData.c_cflag &=~CSTOPB;
    mData.c_cflag &=~CSIZE;
    mData.c_cflag |= CS8;
    mData.c_iflag  = IGNBRK | IGNPAR | INLCR;
    tcflush(mFD, TCIFLUSH);
    tcsetattr(mFD, TCSANOW, &mData);
    return 1;
}

	 
void SetAddr(BYTE newaddr) {

int n;
static int search_addr=0xe0; // start searching for SRF02s here
static int cm02_ver=0;
BYTE sbuf[100];

	if(CommPort==0) {
		fprintf(stderr,"Please Set Com Port.\n");
		return;
	}	

	// Get USB-I2C firmware revision number
	if(cm02_ver==0) {
		printf("Getting firmware revision from USBI2C..."); 
		fflush(stdout);
		sbuf[0] = CM01_CMD; // cmd is for USBI2C
		sbuf[1] = VERSION;  // send version
		sbuf[2] = 0x00;     // two bytes unused (packet==4 bytes)
		sbuf[3] = 0x00;
	   n = write(mFD, &sbuf, 4);
		if (n!=4) {
				  fprintf(stderr,"Wrote %d (instead of 4) bytes...\n",n);
				  perror("get_fw_rev()");
				  return;
		}
		n = read(mFD, &sbuf, 1); // read the rev num
		if (n!=1) {
				  fprintf(stderr,"Read %d (instead of 1) bytes...\n",n);
				  perror("get_fw_rev()");
				  return;
		}
		cm02_ver = sbuf[0];
		printf("USB-I2C rev %i.\n",sbuf[0]);
	}

	Addr=0;
	while(Addr==0) {			// find srf02
		printf("Checking address 0x%02X for SRF02...",search_addr);
		fflush(stdout);
		sbuf[0] = I2C_CMD;		// send cmd to I2C
		sbuf[1] = search_addr+READ;	// addr, read cmd
		sbuf[2] = 0x00;			// srf02 cmd, send ver
		sbuf[3] = 0x01;			// read 1 byte
	   n = write(mFD, &sbuf, 4);
		if (n!=4) {
				  fprintf(stderr,"Wrote %d (instead of 4) bytes...\n",n);
				  return;
		}
		n = read(mFD, &sbuf, 1);
		if (n!=1) {
				  fprintf(stderr,"Read %d (instead of 1) bytes...\n",n);
				  return;
		}
		if(sbuf[0]<20) { // found one!
			printf("found one, revision %i!\n",sbuf[0]);
		   fflush(stdout);
			Addr = search_addr;
		}
		else {
			search_addr += 2;
			printf("\n");
			if(search_addr>0xfe) {
					  printf("\n");
					  search_addr = 0xe0;
			}
		}
	}
	// If already know where SRF02 is, 
		printf("Setting SRF02 address to %02x.\n",newaddr);
		sbuf[0] = CM01_CMD; // cmd is for USBI2C
		sbuf[1] = NEW_ADDRESS;  // set SRF02 address
		sbuf[2] = newaddr;     // last byte unused (packet==4 bytes)
		sbuf[3] = 0x00; 
	   n = write(mFD, &sbuf, 4);
		if (n!=4) {
				  fprintf(stderr,"Wrote %d (instead of 4) bytes...\n",n);
				  return;
		}
		n = read(mFD, &sbuf, 1); // return new address
		if (n!=1) {
				  fprintf(stderr,"Read %d (instead of 1) bytes...\n",n);
				  return;
		}
		else{
				  printf("Address set to %02x.\n",sbuf[0]);
		}
}


int main (int argc, char *argv[]) {
 BYTE newaddr=0x00;
 if (argc!=2) {
			fprintf(stderr,"usage: %s 0xXX, XX hex digits\n",argv[0]);
			return -1;
 }
 else {
			newaddr = strtol(argv[1],(char **)NULL,16);
			switch(newaddr) {
					  case 0xE0:
					  case 0xE2:
					  case 0xE4:
					  case 0xE6:
					  case 0xE8:
					  case 0xEA:
					  case 0xEC:
					  case 0xEE:
					  case 0xF0:
					  case 0xF2:
					  case 0xF4:
					  case 0xF6:
					  case 0xF8:
					  case 0xFA:
					  case 0xFC:
					  case 0xFE:
								 break;
					  default:
								 fprintf(stderr,"Address must be even 0xE0-0xFE\n");
								 return -1;
			}
			printf("setting SRF02 address to %02x\n",newaddr);
 }

 if (SerialOpen("/dev/ttyUSB0",B19200)<0)
	if (SerialOpen("/dev/ttyUSB1",B19200)<0) {
			fprintf(stderr,"Problem opening port");
			return -1;
 }

  SetAddr(newaddr);
 return 0;
} 
